@@ -347,7 +347,8 @@ def visualize_cluster_composition_umap(cluster_data: Dict[int, np.ndarray],
                                      id_dictionary: Optional[Dict[int, str]] = None, 
                                      graph_label = "Community ID",
                                      title = 'UMAP Visualization of Community Compositions',
-                                     neighborhoods: Optional[Dict[int, int]] = None):
+                                     neighborhoods: Optional[Dict[int, int]] = None,
+                                     draw_lines: bool = False):
     """
     Convert cluster composition data to UMAP visualization.
     
@@ -370,6 +371,8 @@ def visualize_cluster_composition_umap(cluster_data: Dict[int, np.ndarray],
     neighborhoods : dict, optional
         Dictionary mapping node IDs to neighborhood IDs {node_id: neighborhood_id}.
         If provided, points will be colored by neighborhood using community coloration methods.
+    draw_lines : bool
+        Whether to draw lines between nodes that share identities (default: False)
     
     Returns:
     --------
@@ -453,15 +456,111 @@ def visualize_cluster_composition_umap(cluster_data: Dict[int, np.ndarray],
     plt.figure(figsize=(12, 8))
     
     if n_components == 2:
-        if use_neighborhood_coloring:
-            scatter = plt.scatter(embedding[:, 0], embedding[:, 1], 
-                                c=point_colors, s=100, alpha=0.7)
-        elif use_identity_coloring:
-            scatter = plt.scatter(embedding[:, 0], embedding[:, 1], 
-                                c=point_colors, s=100, alpha=0.7)
+        # Draw scatter with different markers for multi-identity nodes if draw_lines is enabled
+        if draw_lines:
+            # Separate multi-identity and singleton nodes for different markers
+            singleton_indices = []
+            multi_indices = []
+            singleton_colors = []
+            multi_colors = []
+            
+            for i, cluster_id in enumerate(cluster_ids):
+                vec = cluster_data[cluster_id]
+                if np.sum(vec) > 1:  # Multi-identity
+                    multi_indices.append(i)
+                    multi_colors.append(point_colors[i] if isinstance(point_colors, list) else point_colors)
+                else:  # Singleton
+                    singleton_indices.append(i)
+                    singleton_colors.append(point_colors[i] if isinstance(point_colors, list) else point_colors)
+            
+            # Draw singleton nodes as circles
+            if singleton_indices:
+                if use_neighborhood_coloring or use_identity_coloring:
+                    scatter1 = plt.scatter(embedding[singleton_indices, 0], embedding[singleton_indices, 1], 
+                                         c=singleton_colors, s=100, alpha=0.7, marker='o')
+                else:
+                    scatter1 = plt.scatter(embedding[singleton_indices, 0], embedding[singleton_indices, 1], 
+                                         c=[point_colors[i] for i in singleton_indices], cmap='viridis', s=100, alpha=0.7, marker='o')
+            
+            # Draw multi-identity nodes as squares
+            if multi_indices:
+                if use_neighborhood_coloring or use_identity_coloring:
+                    scatter2 = plt.scatter(embedding[multi_indices, 0], embedding[multi_indices, 1], 
+                                         c=multi_colors, s=100, alpha=0.7, marker='s')
+                else:
+                    scatter2 = plt.scatter(embedding[multi_indices, 0], embedding[multi_indices, 1], 
+                                         c=[point_colors[i] for i in multi_indices], cmap='viridis', s=100, alpha=0.7, marker='s')
+                scatter = scatter2  # For colorbar reference
+            else:
+                scatter = scatter1 if singleton_indices else None
         else:
-            scatter = plt.scatter(embedding[:, 0], embedding[:, 1], 
-                                c=point_colors, cmap='viridis', s=100, alpha=0.7)
+            # Original behavior when draw_lines is False
+            if use_neighborhood_coloring:
+                scatter = plt.scatter(embedding[:, 0], embedding[:, 1], 
+                                    c=point_colors, s=100, alpha=0.7)
+            elif use_identity_coloring:
+                scatter = plt.scatter(embedding[:, 0], embedding[:, 1], 
+                                    c=point_colors, s=100, alpha=0.7)
+            else:
+                scatter = plt.scatter(embedding[:, 0], embedding[:, 1], 
+                                    c=point_colors, cmap='viridis', s=100, alpha=0.7)
+        
+        # Draw lines between nodes with shared identities (only if draw_lines=True)
+        if draw_lines:
+            # First pass: identify unique multi-identity configurations and their representatives
+            multi_config_map = {}  # Maps tuple(config) -> {'count': int, 'representative_idx': int}
+            
+            for i, cluster_id in enumerate(cluster_ids):
+                vec = cluster_data[cluster_id]
+                if np.sum(vec) > 1:  # Multi-identity node
+                    config = tuple(vec)  # Convert to hashable tuple
+                    if config not in multi_config_map:
+                        multi_config_map[config] = {'count': 1, 'representative_idx': i}
+                    else:
+                        multi_config_map[config]['count'] += 1
+            
+            # Second pass: draw lines for each unique configuration
+            for config, info in multi_config_map.items():
+                i = info['representative_idx']
+                count = info['count']
+                vec1 = np.array(config)
+                
+                # For each identity this configuration has, find the closest representative
+                identity_indices = np.where(vec1 == 1)[0]
+                
+                for identity_idx in identity_indices:
+                    best_target = None
+                    best_distance = float('inf')
+                    backup_target = None
+                    backup_distance = float('inf')
+                    
+                    # Find closest node with this specific identity
+                    for j, cluster_id2 in enumerate(cluster_ids):
+                        if i != j:  # Don't connect to self
+                            vec2 = cluster_data[cluster_id2]
+                            if vec2[identity_idx] == 1:  # Shares this specific identity
+                                distance = np.linalg.norm(embedding[i] - embedding[j])
+                                
+                                # Prefer singleton nodes
+                                if np.sum(vec2) == 1:  # Singleton
+                                    if distance < best_distance:
+                                        best_distance = distance
+                                        best_target = j
+                                else:  # Multi-identity node (backup)
+                                    if distance < backup_distance:
+                                        backup_distance = distance
+                                        backup_target = j
+                    
+                    # Draw line to best target (prefer singleton, fallback to multi)
+                    target = best_target if best_target is not None else backup_target
+                    if target is not None:
+                        # Calculate relative line weight with reasonable cap
+                        max_count = max(info['count'] for info in multi_config_map.values())
+                        relative_weight = count / max_count  # Normalize to 0-1
+                        line_weight = 0.3 + relative_weight * 1.2  # Scale to 0.3-1.5 range
+                        plt.plot([embedding[i, 0], embedding[target, 0]], 
+                               [embedding[i, 1], embedding[target, 1]], 
+                               alpha=0.3, color='gray', linewidth=line_weight)
         
         if label:
             # Add cluster ID labels
@@ -516,15 +615,112 @@ def visualize_cluster_composition_umap(cluster_data: Dict[int, np.ndarray],
         fig = plt.figure(figsize=(14, 10))
         ax = fig.add_subplot(111, projection='3d')
         
-        if use_neighborhood_coloring:
-            scatter = ax.scatter(embedding[:, 0], embedding[:, 1], embedding[:, 2],
-                               c=point_colors, s=100, alpha=0.7)
-        elif use_identity_coloring:
-            scatter = ax.scatter(embedding[:, 0], embedding[:, 1], embedding[:, 2],
-                               c=point_colors, s=100, alpha=0.7)
+        # Draw scatter with different markers for multi-identity nodes if draw_lines is enabled
+        if draw_lines:
+            # Separate multi-identity and singleton nodes for different markers
+            singleton_indices = []
+            multi_indices = []
+            singleton_colors = []
+            multi_colors = []
+            
+            for i, cluster_id in enumerate(cluster_ids):
+                vec = cluster_data[cluster_id]
+                if np.sum(vec) > 1:  # Multi-identity
+                    multi_indices.append(i)
+                    multi_colors.append(point_colors[i] if isinstance(point_colors, list) else point_colors)
+                else:  # Singleton
+                    singleton_indices.append(i)
+                    singleton_colors.append(point_colors[i] if isinstance(point_colors, list) else point_colors)
+            
+            # Draw singleton nodes as circles
+            if singleton_indices:
+                if use_neighborhood_coloring or use_identity_coloring:
+                    scatter1 = ax.scatter(embedding[singleton_indices, 0], embedding[singleton_indices, 1], embedding[singleton_indices, 2],
+                                        c=singleton_colors, s=100, alpha=0.7, marker='o')
+                else:
+                    scatter1 = ax.scatter(embedding[singleton_indices, 0], embedding[singleton_indices, 1], embedding[singleton_indices, 2],
+                                        c=[point_colors[i] for i in singleton_indices], cmap='viridis', s=100, alpha=0.7, marker='o')
+            
+            # Draw multi-identity nodes as squares
+            if multi_indices:
+                if use_neighborhood_coloring or use_identity_coloring:
+                    scatter2 = ax.scatter(embedding[multi_indices, 0], embedding[multi_indices, 1], embedding[multi_indices, 2],
+                                        c=multi_colors, s=100, alpha=0.7, marker='s')
+                else:
+                    scatter2 = ax.scatter(embedding[multi_indices, 0], embedding[multi_indices, 1], embedding[multi_indices, 2],
+                                        c=[point_colors[i] for i in multi_indices], cmap='viridis', s=100, alpha=0.7, marker='s')
+                scatter = scatter2  # For colorbar reference
+            else:
+                scatter = scatter1 if singleton_indices else None
         else:
-            scatter = ax.scatter(embedding[:, 0], embedding[:, 1], embedding[:, 2],
-                               c=point_colors, cmap='viridis', s=100, alpha=0.7)
+            # Original behavior when draw_lines is False
+            if use_neighborhood_coloring:
+                scatter = ax.scatter(embedding[:, 0], embedding[:, 1], embedding[:, 2],
+                                   c=point_colors, s=100, alpha=0.7)
+            elif use_identity_coloring:
+                scatter = ax.scatter(embedding[:, 0], embedding[:, 1], embedding[:, 2],
+                                   c=point_colors, s=100, alpha=0.7)
+            else:
+                scatter = ax.scatter(embedding[:, 0], embedding[:, 1], embedding[:, 2],
+                                   c=point_colors, cmap='viridis', s=100, alpha=0.7)
+        
+        # Draw lines between nodes with shared identities (only if draw_lines=True)
+        if draw_lines:
+            # First pass: identify unique multi-identity configurations and their representatives
+            multi_config_map = {}  # Maps tuple(config) -> {'count': int, 'representative_idx': int}
+            
+            for i, cluster_id in enumerate(cluster_ids):
+                vec = cluster_data[cluster_id]
+                if np.sum(vec) > 1:  # Multi-identity node
+                    config = tuple(vec)  # Convert to hashable tuple
+                    if config not in multi_config_map:
+                        multi_config_map[config] = {'count': 1, 'representative_idx': i}
+                    else:
+                        multi_config_map[config]['count'] += 1
+            
+            # Second pass: draw lines for each unique configuration
+            for config, info in multi_config_map.items():
+                i = info['representative_idx']
+                count = info['count']
+                vec1 = np.array(config)
+                
+                # For each identity this configuration has, find the closest representative
+                identity_indices = np.where(vec1 == 1)[0]
+                
+                for identity_idx in identity_indices:
+                    best_target = None
+                    best_distance = float('inf')
+                    backup_target = None
+                    backup_distance = float('inf')
+                    
+                    # Find closest node with this specific identity
+                    for j, cluster_id2 in enumerate(cluster_ids):
+                        if i != j:  # Don't connect to self
+                            vec2 = cluster_data[cluster_id2]
+                            if vec2[identity_idx] == 1:  # Shares this specific identity
+                                distance = np.linalg.norm(embedding[i] - embedding[j])
+                                
+                                # Prefer singleton nodes
+                                if np.sum(vec2) == 1:  # Singleton
+                                    if distance < best_distance:
+                                        best_distance = distance
+                                        best_target = j
+                                else:  # Multi-identity node (backup)
+                                    if distance < backup_distance:
+                                        backup_distance = distance
+                                        backup_target = j
+                    
+                    # Draw line to best target (prefer singleton, fallback to multi)
+                    target = best_target if best_target is not None else backup_target
+                    if target is not None:
+                        # Calculate relative line weight with reasonable cap
+                        max_count = max(info['count'] for info in multi_config_map.values())
+                        relative_weight = count / max_count  # Normalize to 0-1
+                        line_weight = 0.3 + relative_weight * 1.2  # Scale to 0.3-1.5 range
+                        ax.plot([embedding[i, 0], embedding[target, 0]], 
+                               [embedding[i, 1], embedding[target, 1]], 
+                               [embedding[i, 2], embedding[target, 2]], 
+                               alpha=0.3, color='gray', linewidth=line_weight)
         
         if label:
             # Add cluster ID labels
